/*
 * Lakeshore 330 monitor 
 * through a serial port
 * 20071114 iwata
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include "rs232c.h"
#include "sio_server.h"
#include "time.h"

#define LOGFILE "lakeshore.log"
#define INTERVAL 45

FILE *fp;

void ending(int fd) {
  fclose(fp);
  close_rs232c(fd);

  exit(EXIT_SUCCESS);
}

int main(int argc, char *argv[]) {
  int fd;
  char command[256];
  char ret[256]="";
  char cp; //rch[64];
  int mode = 0; /* 0=monitor mode 1=command mode */
  float tempa, tempb;
  myTime ctime;     /* (current) time */
  double jd, jd0 = -1.0;
  char str[20];

  // receive SIGINT
  (void)signal(SIGINT, ending);

  if(argc==2) {
    sprintf(command,"%s\n",argv[1]);
    printf(command,"%s\n",argv[1]);
    mode = 1;
  } else if (argc > 2) {
    printf("usage : %s [command]\n",argv[0]);
    exit(EXIT_FAILURE);
  }

  set_lt(9); // time.c

  fd=init_rs232c();
  if(fd<0)
    exit(EXIT_FAILURE);
  
  if (mode) {
    if(sendstr_rs232c(fd,command)==ERROR)
      exit(EXIT_FAILURE);
    sleep(2);
    while(recievechar_rs232c(fd,&cp)!=ERROR) {
      strncat(ret,&cp,1);
      if(cp=='\n'||cp=='\0')
	break;
    }
    printf("received:%s",ret);
  } else {
    // monitor mode
    while (1) {
      if ((fp = fopen(LOGFILE, "a"))==NULL) {
	printf("error in opening LOGFILE");
	exit(EXIT_FAILURE);
      }
    
      strcpy(command, "SCHN A\n");
      if(sendstr_rs232c(fd,command)==ERROR)
	exit(EXIT_FAILURE);
      printf("finished sending %s", command);
      sleep(2);
      strcpy(command, "SDAT?\n");
      if(sendstr_rs232c(fd,command)==ERROR)
	exit(EXIT_FAILURE);
      printf("finished sending %s", command);
      sleep(2);
      while(recievechar_rs232c(fd,&cp)!=ERROR) {
	strncat(ret,&cp,1);
	if(cp=='\n'||cp=='\0')
	  break;
      }
      recievechar_rs232c(fd,&cp); // clearing CR
      printf("received:%s",ret);
      tempa = atof(ret);
      strcpy(ret,"\0");
      sleep(2);

      strcpy(command, "SCHN B\n");
      if(sendstr_rs232c(fd,command)==ERROR)
	exit(EXIT_FAILURE);
      printf("finished sending %s", command);
      sleep(2);
      strcpy(command, "SDAT?\n");
      if(sendstr_rs232c(fd,command)==ERROR)
	exit(EXIT_FAILURE);
      printf("finished sending %s", command);
      sleep(2);
      while(recievechar_rs232c(fd,&cp)!=ERROR) {
	strncat(ret,&cp,1);
	if(cp=='\n'||cp=='\0')
	  break;
      }
      recievechar_rs232c(fd,&cp); // clearing CR
      printf("received:%s",ret);
      tempb = atof(ret);
      strcpy(ret,"\0");
  
      get_localtime(&ctime);
      jd = myCalcmjd(ctime.year, ctime.month, ctime.day, 
		     ctime.hour, ctime.min, ctime.sec);
      if (jd0<0) {
	jd0 = jd;
      }

      sprintf(str, "%04d.%02d.%02d-%02d:%02d:%02d", 
	      ctime.year, 
	      ctime.month,
	      ctime.day,
	      ctime.hour,
	      ctime.min,
	      ctime.sec);

      printf("%s %12.6f", str, (jd-jd0)*24.0);
      printf("%7.2f %7.2f\n", tempa, tempb);
      fprintf(fp, "%s %12.6f", str, (jd-jd0)*24.0);
      fprintf(fp,"%7.2f %7.2f\n", tempa, tempb);

      fflush(fp);
      sleep(INTERVAL);
    }
  }



  exit(EXIT_SUCCESS);
}

