#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <termios.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <fcntl.h>
#include "sio_server.h"

#define	SIOPORT	"/dev/ttyS1"

int init_rs232c()
{
	int fd;
	struct termios	tio;

#ifdef DEBUG
	printf("init_rs232c()\n");
#endif
	if((fd=open(SIOPORT,O_RDWR))<0)
	{
		perror("open");
		return ERROR;
	}

	if((int)tcgetattr(fd,&tio)<0)
	{
		perror("tcgetattr");
		return ERROR;
	}

	tio.c_cflag |= B1200;
	cfsetispeed(&tio,B1200);
	cfsetospeed(&tio,B1200);
 
	tio.c_cflag &= ~CSIZE;
	tio.c_cflag |= CS7;
	tio.c_cflag |= PARENB;
	tio.c_cflag |= PARODD;
	tio.c_cflag &= ~CSTOPB;

	tio.c_cflag |= (CLOCAL | CREAD);
	tio.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);

	if((int)tcsetattr(fd,TCSANOW,&tio)<0)
	{
		perror("tcsetattr");
		return ERROR;
	}

	if((int)tcgetattr(fd,&tio)<0)
	{
		perror("tcgetattr");
		return ERROR;
	}

	return fd;
}

int close_rs232c(int fd)
{
#ifdef DEBUG
	printf("close_rs232c()\n");
#endif
	if(close(fd)<0)
	{
		perror("close");
		return ERROR;
	}

	return OK;
}

int sendchar_rs232c(int fd, char ch)
{
#ifdef DEBUG
	printf("sendchar_rs232c()\n");
#endif
	if(write(fd,&ch,1)<0)
	{
		perror("write");
		return ERROR;
	}

	return OK;
}

int sendstr_rs232c(int fd, char *str)
{
	int	n,nwritten;
	char	*com;

#ifdef DEBUG
	printf("sendstr_rs232c()\n");
#endif
	n=strlen(str);
	com = (char *)malloc(sizeof(char)*(n+2));

	if(strchr(str,'\n')==NULL)
		sprintf(com,"%s\n",str);
	else
		strcpy(com,str);
//	com[n]='\0'; // '\n' = 0x0a
//	com[n+1]='\0';
#ifdef DEBUG
	printf("command: %s",com);
#endif

	nwritten = write(fd,com,n+2);
	if(nwritten<0)
	{
		perror("write");
		return ERROR;
	}

	return nwritten;
}

int recievechar_rs232c(int fd, char *ch)
{
  fd_set rset;
  struct timeval tv;

#ifdef DEBUG
  printf("recievechar_rs232c()\n");
#endif
  FD_ZERO(&rset);
  FD_SET(fd,&rset);

  tv.tv_sec = SIO_WAIT;
  tv.tv_usec = 0;
  if(0<select(FD_SETSIZE,&rset,NULL,NULL,&tv)) {
    //  if(0<select(FD_SETSIZE,&rset,NULL,NULL,NULL)) {
    if(FD_ISSET(fd,&rset)) {
      if(read(fd,ch,1)<0) {
	perror("read");
	return ERROR;
      }
    }
  } else {
    fprintf(stderr,"recievechar_rs232c(): SIO timed out\n");
    return ERROR;
  }
  
  return OK;
}

int recievestr_rs232c(int fd, char *ch, int nchar)
{
  int nread;
  fd_set rset;
  char cp;
  struct timeval tv;

#ifdef DEBUG
  printf("recievestr_rs232c()\n");
#endif
  FD_ZERO(&rset);
  FD_SET(fd,&rset);

  tv.tv_sec = SIO_WAIT;
  tv.tv_usec = 0;
  if(0<select(FD_SETSIZE,&rset,NULL,NULL,&tv)) {
    //  if(0<select(FD_SETSIZE,&rset,NULL,NULL,NULL)) {
    if(FD_ISSET(fd,&rset)) {
      nread=0;
      ch[0]='\0';
      while(nread<nchar) {
	if(read(fd,&cp,1)<0) {
	  perror("read");
	  return ERROR;
	}
	if(cp=='\n')
	  break;
	sprintf(ch,"%s%c",ch,cp);
      }
    }
  } else {
    fprintf(stderr,"recievechar_rs232c(): SIO timed out\n");
    return ERROR;
  }
  
  return nread;
}
				

